<?php
    /**
     * ppaste - Quick paste pictures anytime anywhere
     *
     * @author Bela Hausmann <post@belahausmann.name>
     * @link http://bitbucket.org/and3k/ppaste/
     * @version 0.3-rc1
     * @copyright Copyright (c) 2009 Bela Hausmann, see README
     * @license http://opensource.org/licenses/agpl-v3.html GNU Affero General Public License version 3
     */

    /**
      * DEFAULT CONFIG:
      */

    $PPASTE                     = true; # Do not change this!
    $PPASTE_VERSION             = '0.3-rc1';
    $PPASTE_FLAVOUR             = 'ppaste';

    $PPASTE_PICTURE_DIR         = 'p/';
    $PPASTE_PICTURE_URL         = 'p/';
    $PPASTE_CODE_LENGTH         = 8;

    $PPASTE_RENAME_FUNCTION     = 'rename_ppaste';

    $PPASTE_IMAGE_FUNCTION      = 'image_phpthumb';
    $PPASTE_IMAGE_LIB_PATH      = '';
    $PPASTE_IMAGE_SIZE          = array(640, 480);

    $PPASTE_SHOW_TAIL           = true;
    $PPASTE_SHOW_FILETYPES      = true;
    $PPASTE_SHOW_BOOKMARKLET    = true;

    $PPASTE_URL                 = $_SERVER['PHP_SELF'];
    $PPASTE_COMPLETE_URL        = (isset($_SERVER['HTTPS']) ? 'https' : 'http') . '://' . $_SERVER['SERVER_NAME'] . $PPASTE_URL;

    // Reserved (not implemented) settings
    $PPASTE_TTL                 = 10000; // Time to live in minutes (~7d)
    $PPASTE_TTL_HOLD            = 50000; // Time to live in minutes, when Hold is activated (~5w), see dpaste.com

    is_file('config.php') and include('config.php');

    /**
      * CHECK ENV AND SETTINGS:
      */

    ini_get('allow_url_fopen') or die('Please switch <i>allow_url_fopen</i> to <i>On</i> in your <code>php.ini</code> to allow grabing pictures via HTTP');
    if($PPASTE_IMAGE_FUNCTION != 'image_dummy')
    {
        $PPASTE_IMAGE_LIB_PATH or die('Please set <code>$PPASTE_IMAGE_LIB_PATH</code> or disable image manipulation by setting <code>$PPASTE_IMAGE_FUNCTION</code> to <code>\'image_dummy\'</code>. Not recoding images after uploading may be a security risk!');
        require_once $PPASTE_IMAGE_LIB_PATH;
    }

    is_dir($PPASTE_PICTURE_DIR) or die('Please create upload directory: <code>mkdir /path/to/your/ppaste/install/<b>' . $PPASTE_PICTURE_DIR . '</b></code>');
    is_writable($PPASTE_PICTURE_DIR) or die('Please make upload directory writable: <code>chmod 0777 /path/to/your/ppaste/install/<b>' . $PPASTE_PICTURE_DIR . '</b></code>');
    if(!is_file($PPASTE_PICTURE_DIR . '.htaccess')) file_put_contents($PPASTE_PICTURE_DIR . '.htaccess', 'php_value engine off'); # Disable php for extra security.

    /**
     * FUNCTIONS:
     */

    /**
     * No actual renaming is done. This might be a security risk.
     * ONLY USE IF YOU KNOW WHAT YOU ARE DOING!
     */
    function rename_dummy($filename, $mimetype) { return $filename; }

    /**
     * Standard ppaste renaming function
     */
    function rename_ppaste($filename, $mimetype)
    {
        # strip extension(s)
        $nameparts = explode('.', $filename, 2);
        $name = $nameparts[0];

        # remove special chars
        $name = preg_replace('/[^a-zA-Z0-9\-_]+/', '_', $name);
        # no leading or trailing underscores
        $name = preg_replace('/(^_+)|(_+$)/', '', $name);

        # convert to lower case (disabled for now)
        #$name = strtolower($name);

        # no empty/"hidden" files and no filenames with only special chars
        if($name === '') throw new Exception('Empty filename');

        # fixed extension based on minetype
        switch($mimetype)
        {
            case 'image/jpeg': $ext = 'jpg'; break;
            case 'image/png': $ext = 'png'; break;
            case 'image/gif': $ext = 'gif'; break;
            default: throw new Exception('Invalid MIME-Type in function rename_ppaste($filename, $mimetype): ' . $mimetype);
        }
        return $name . '.' . $ext;
    }

    /**
     * No actual image manipulation is done. This might be a security risk.
     * ONLY USE IF YOU KNOW WHAT YOU ARE DOING!
     */
    function image_dummy($source, $destination) { copy($source, $destination); }

    /**
     * Autoresizing uploaded images by use of the PhpThumb libray (http://phpthumb.gxdlabs.com/)
     */
    function image_phpthumb($source, $destination)
    {
        global $PPASTE_IMAGE_SIZE;
        # Catch warnings if corrupted images are uploaded.
        @$thumb = PhpThumbFactory::create($source);
        @$thumb->resize($PPASTE_IMAGE_SIZE[0], $PPASTE_IMAGE_SIZE[1])->save($destination);
    }

    if(isset($PPASTE_CONFIG_INC) and $PPASTE_CONFIG_INC) return;

    /**
      * MAIN PART:
      */
    #print_r($_FILES);
    #print_r($_GET);
    #print_r($_POST);
    #print_r($_SERVER);

    if(isset($_FILES['file']) and $_FILES['file']['error'] !=  UPLOAD_ERR_NO_FILE) $mode = 'upload';
    elseif(isset($_GET['grab']) and $_GET['grab'] != '') $mode = 'get';
    else $mode = false;
    if($mode)
        try
        {
            if($mode == 'upload')
            {
                if($_FILES['file']['error']) throw new Exception('File upload error');
                if($_FILES['file']['size'] == 0) throw new Exception('Empty file');
                $filelocation = $_FILES['file']['tmp_name'];
                $filename = $_FILES['file']['name'];
            }
            if($mode == 'get')
            {
                $filelocation = $_GET['grab'];
                $filename = basename($filelocation);
            }
            $imagedata = @getimagesize($filelocation); # Get MIME-Type
            if(!$imagedata) throw new Exception('Not a valid image');
            #print_r($imagedata);
            $mimetype = $imagedata['mime'];
            if($mode == 'upload' and $mimetype != $_FILES['file']['type']) throw new Exception('MIME-Type mismatch'); # Happens sometimes with mixed up extensions
            if(!(
                   $mimetype == 'image/jpeg'
                or $mimetype == 'image/png'
                or $mimetype == 'image/gif'
                )) throw new Exception('Picture type <code>' . $mimetype . '</code> not allowed');

            $newfilename = call_user_func($PPASTE_RENAME_FUNCTION, $filename, $mimetype);

            $code = '';
            while(is_dir($PPASTE_PICTURE_DIR . $code)) $code = substr(md5(rand()), 0, $PPASTE_CODE_LENGTH);
            mkdir($PPASTE_PICTURE_DIR . $code);

            $tmpfile = $PPASTE_PICTURE_DIR . $code . '/.tmp';
            copy($filelocation, $tmpfile);
            call_user_func($PPASTE_IMAGE_FUNCTION, $tmpfile, $PPASTE_PICTURE_DIR . $code . '/' . $newfilename);
            unlink($tmpfile);

            $link = $PPASTE_PICTURE_URL . $code . '/' . $newfilename;
            $message = array(
                'class' => 'success',
                'text' => 'Success: <a href="' . $link . '">Link to your picture</a>'
                );
        }
        catch(Exception $e)
        {
            $message = array(
                'class' => 'failure',
                'text' => 'Error: ' . $e->getMessage()
                );
        }
    else $message = null;

    /**
     * OUTPUT:
     */
    echo '<?xml version="1.0" encoding="UTF-8"?>'."\n";
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
    <head>
        <title><?php echo $PPASTE_FLAVOUR ?></title>
        <link rel="stylesheet" type="text/css" href="ppaste.css" />
    </head>

    <body>
        <div id="body">
            <h1><?php echo $PPASTE_FLAVOUR ?></h1>
            <div id="content">
                <h2>Add picture</h2>
                <?php if($message) echo '<p class="' . $message['class'] . '">' . $message['text'] . '</p>' ?>
                <form action="<?php echo $PPASTE_URL ?>" method="post" enctype="multipart/form-data">
                    <fieldset>
                        <legend>Upload local picture</legend>
                        <p>
                            <input type="file" name="file" />
                            <input type="submit" value="Upload" />
                        </p>
                    </fieldset>
                </form>
                <form action="<?php echo $PPASTE_URL ?>" method="get">
                    <fieldset>
                        <legend>Grab picture from URL</legend>
                        <p>
                            <input type="text" name="grab" />
                            <input type="submit" value="Grab" />
                        </p>
                    </fieldset>
                </form>
                <?php if($PPASTE_SHOW_FILETYPES) { ?>
                    <p><small>Allowed filetypes: JPEG, PNG, GIF</small></p>
                <?php } ?>
                <?php if(is_file('about.html')) echo file_get_contents('about.html') ?>
                <?php if($PPASTE_SHOW_BOOKMARKLET) { ?>
                    <p><?php echo $PPASTE_FLAVOUR ?> Bookmarklet <small>(<a href="http://en.wikipedia.org/wiki/Bookmarklet">?</a>)</small>: <a href="javascript:location.href='<?php echo $PPASTE_COMPLETE_URL ?>?grab='+encodeURIComponent(location.href)">Post to <?php echo $PPASTE_FLAVOUR ?></a></p>
                <?php } ?>
            </div>
            <?php if($PPASTE_SHOW_TAIL) { ?>
                <div id="tail">
                    <hr />
                    <p>
                        <a href="http://bitbucket.org/and3k/ppaste/">ppaste</a> v<?php echo $PPASTE_VERSION ?> - Quick paste pictures anytime anywhere - All pictures may be deleted after some time
                    </p>
                </div>
            <?php } ?>
        </div>
    </body>
</html>
